=== modified file 'loggerhead/TODO'
--- loggerhead/TODO	2006-12-20 21:24:32 +0000
+++ loggerhead/TODO	2006-12-20 22:00:14 +0000
@@ -5,12 +5,10 @@
 
 * side by side diff?  matty says "overflow-x" css property for horiz scrollbar
 
-* make equivalent of start page:
-    http://goffredo-baroncelli.homelinux.net/bazaar
-
-* on revision page: view/download patch
-
 * potentially do a 'bzr pull' on a branch on an interval timer
 
 * give this thing a version # or something
 * make a loggerhead webpage
+
+* it would be cool to view 1 revision relative to another base, and view the
+  diff & download the bundle for that

=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2006-12-20 21:17:53 +0000
+++ loggerhead/branchview.py	2006-12-20 22:00:14 +0000
@@ -36,6 +36,7 @@
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.bundle_ui import BundleUI
 
 
 with_history_lock = util.with_lock('_history_lock', 'History')
@@ -58,6 +59,7 @@
         self.annotate = AnnotateUI(self)
         self.download = DownloadUI(self)
         self.atom = AtomUI(self)
+        self.bundle = BundleUI(self)
         
         # force history object to be loaded:
         self.get_history()

=== added file 'loggerhead/controllers/bundle_ui.py'
--- loggerhead/controllers/bundle_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/bundle_ui.py	2006-12-20 22:00:14 +0000
@@ -0,0 +1,56 @@
+#
+# Copyright (C) 2006  Robey Pointer <robey@lag.net>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import os
+import time
+
+import turbogears
+from cherrypy import HTTPRedirect, response
+
+from loggerhead import util
+
+
+class BundleUI (object):
+
+    def __init__(self, branch):
+        # BranchView object
+        self._branch = branch
+        self.log = branch.log
+
+    @turbogears.expose()
+    def default(self, *args, **kw):
+        # /bundle/<rev_id>/filename
+        z = time.time()
+        h = self._branch.get_history()
+
+        if len(args) < 1:
+            raise HTTPRedirect(self._branch.url('/changes'))
+        revid = h.fix_revid(args[0])
+
+        try:
+            bundle_data = h.get_bundle(revid)
+        except Exception, x:
+            self.log.error('Exception fetching bundle: %s' % (x,))
+            util.log_exception(self.log)
+            raise HTTPRedirect(self._branch.url('/changes'))
+            
+        response.headers['Content-Type'] = 'text/plain'
+        response.headers['Content-Length'] = len(bundle_data)
+        response.body = bundle_data
+        self.log.info('/bundle: %r seconds' % (time.time() - z,))
+        return response.body

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2006-12-20 09:29:42 +0000
+++ loggerhead/controllers/revision_ui.py	2006-12-20 22:00:14 +0000
@@ -28,8 +28,6 @@
 
 from loggerhead import util
 
-log = logging.getLogger("loggerhead.controllers")
-
 
 class RevisionUI (object):
 

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2006-12-20 21:17:53 +0000
+++ loggerhead/history.py	2006-12-20 22:00:14 +0000
@@ -46,6 +46,7 @@
 import bzrlib
 import bzrlib.annotate
 import bzrlib.branch
+import bzrlib.bundle.serializer
 import bzrlib.diff
 import bzrlib.errors
 import bzrlib.progress
@@ -798,3 +799,16 @@
             lineno += 1
         
         self.log.debug('annotate: %r secs' % (time.time() - z,))
+
+    @with_branch_lock
+    @with_bzrlib_read_lock
+    def get_bundle(self, revid):
+        parents = self._revision_graph[revid]
+        if len(parents) > 0:
+            parent_revid = parents[0]
+        else:
+            parent_revid = None
+        s = StringIO()
+        bzrlib.bundle.serializer.write_bundle(self._branch.repository, revid, parent_revid, s)
+        return s.getvalue()
+

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2006-12-20 09:29:42 +0000
+++ loggerhead/templates/revision.kid	2006-12-20 22:00:14 +0000
@@ -18,6 +18,7 @@
 	<div class="links">
 	    <div> <b>&#8594;</b> <a href="${branch.url([ '/files', revid ])}">browse files</a> </div>
 	    <div> <b>&#8594;</b> <a href="${branch.url('/changes', start_revid=revid)}">view branch changes</a> </div>
+	    <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, 'bundle.txt' ])}">view/download patch</a> </div>
 	</div>
 </h1>
  

